<?php defined('SYSPATH') or die('No direct script access.');
/**
 * class Controller_admin_service
 *
 * Odpowiada za zarządzanie usługami
 */
class Controller_admin_service extends Controller_admin
{	

	/**
	 * Konstruktor...
	 * 
	 */
	public function __construct()
	{
		parent::__construct('acp_service');
	}
	
	/**
	 * Box dodawania usułgi...
	 *
	 * @access public
	 */
	public function action_add_box()
	{
		Core::ajax()->assign('title',  'Dodawanie usułgi');
		Core::ajax()->assign('message', Core::view()->execute('admin/service/add', TRUE));
		Core::ajax()->send();
	}
	
	/**
	 * Box edytowania usługi
	 *
	 * @access public
	 */
	public function action_edit_box()
	{
		$id   = Core::request()->post('id');
		$data = Service::get($id);
		
		if(count($data) > 1)
		{
			Core::ajax()->assign('title', 'Edytowanie usługi: ' . $data['name']);
			Core::view()->assign('data', $data);
			Core::ajax()->assign('message', Core::view()->execute('admin/service/edit', TRUE));
		}
		else
		{
			Core::ajax()->assign('title', 'Błąd');
			Core::ajax()->assign('message', '<div class="error">Nie znaleziono takiej usługi</div>');
		}
		Core::ajax()->send();
	}
	
	/**
	 * Dodaje...
	 *
	 * @access public
	 */
	public function action_add()
	{
		$form = Core::request()->post();
		$form = $this->parse($form);
		
		// Dodajmy...
		$result = Service::add($form);
		
		Core::ajax()->assign('result',  $result[0] ? 'success' : 'error');
		Core::ajax()->assign('message', $result[1]);
		
		if($result[0])
		{
			Logs::add("Dodał usługę {$form['name']}", Member::logged(), 'service');
		}
		
		Core::ajax()->send();
	}
	
	/**
	 * Edytuje usługę
	 *
	 *
	 * @access public
	 */
	public function action_edit()
	{
		$form = Core::request()->post();
		$form = $this->parse($form);
		
		// Dodajmy...
		$result = Service::edit($form);
		
		Core::ajax()->assign('result',  $result[0] ? 'success' : 'error');
		Core::ajax()->assign('message', $result[1]);
		
		if($result[0])
		{
			Logs::add("Edytował usługę {$form['name']}", Member::logged(), 'service');
		}
		
		Core::ajax()->send();
	}
	
	/**
	 * Usuwa usługę...
	 *
	 *
	 * @access public
	 */
	public function action_delete()
	{
		$id = Core::request()->post('id');
		
		// Dodajmy...
		$result = Service::delete($id);
		
		Core::ajax()->assign('result',  $result[0] ? 'success' : 'error');
		Core::ajax()->assign('message', $result[1]);
		
		if($result[0])
		{
			$name = Core::request()->post('name');
			Logs::add("Usunął usługę {$name}", Member::logged(), 'service');
		}
		
		Core::ajax()->send();
	}
	
	/**
	 * Parsuje odpowiednie elementy formularza
	 *
	 * @param array
	 *
	 * @access private
	 * @return array
	 */
	private function parse($form)
	{
		// AmxBans ID i Server ID
		list($form['amxbans_id'], $form['server_id']) = explode('|', $form['amxbans_id']);
		
		// Grupy, odzielmy je przecinkami...
		$form['allowed_group'] = implode(',', $form['allowed_group']);
		
		return $form;
	}
	
	/**
	 * Usuwa usługę już zakupioną
	 *
	 * @access public
	 */
	public function action_delete_from_buy_cache()
	{
		$data    = Service::getFromBuyCache( Core::request()->post('id') );
		
		$amxbans = Core::amxbans()->connect(Core::amxbans()->getAmxBans($data['amxbans_id']));
		
		if($data['status'])
		{
			if($amxbans)
			{
				$amxbans->usunFlagi($data['amxbans_member_id'], $data['flags']);
				$_result = 'Usunięto flagi użytkownika ' . Member::getDisplayName($data['member_id']) . ' dla tej usługi';
				
				Service::updateFromBuyCache(Core::request()->post('id'), 'status', 0);
				Service::updateFromBuyCache(Core::request()->post('id'), 'expired', time());
			}
			else
			{
				$_result = 'Nie można nawiązać połączenia z amxbansem tej usługi';
			}
		}
		else
		{
			$_result = 'Ta usługa została już wyłączona';
		}
		
		Core::ajax()->assign('message', $_result);
		Core::ajax()->send();
	}
	
	/**
	 * Przedłuża usługę
	 *
	 * @access public
	 */
	public function action_extend()
	{
		$id   = Core::request()->post('id');
		$day  = Core::request()->post('day');
		
		if(!is_numeric($day))
		{
			$message = 'Niepoprawna liczba dni';
		}
		else
		{
			$sdata = Service::getFromBuyCache($id);
			if(!$sdata)
			{
				$message = 'Taka usługa nie istnieje';
			}
			else
			{
				Service::insertBuy($sdata['member_id'], $sdata['service_id'], $sdata['amxbans_member_id'], $day);
				$message = 'Usługa użytkownika ' . Member::getDisplayName($sdata['member_id']) . ' została przedłużona';
			}
		}
		
		Core::ajax()->assign('message', $message);
		Core::ajax()->send();
	}
}