<?php defined('SYSPATH') or die('No direct script access.');

abstract class News
{
	
	/**
	 * Zwraca ilość newsów w bazie danych
	 *
	 * @access static
	 * @return integer
	 */
	static public function getCount()
	{
		$q = Core::db()->query('SELECT COUNT(`id`) AS `count` FROM `news_`')->fetch();
		return $q['count'];
	}
	
	/**
	 * Zwraca ilość newsów od podanego rekordku
	 *
	 * @param integer
	 * @param integer
	 *
	 * @access static
	 * @return array
	 */
	static public function get($start, $rows)
	{
		$q = Core::db()->query('
			SELECT a. * , b.`rep_points` , b.`rep_cache` , b.`section` , b.`section_id`
				FROM `news_` a LEFT JOIN `reputation_cache_` b ON b.`section_id` = a.`id` AND b.`section` = \'news\' order by a.`id` DESC LIMIT '.intval($start).', '.intval($rows));
		return $q->fetchAll();
	}
	
	/**
	 * Zwraca konkretnego newsa
	 *
	 * @param integer
	 *
	 * @access static
	 * @return array
	 */
	static public function getOne($id)
	{
		$stmt = Core::db()->prepare('SELECT * FROM `news_` where `id` = ? LIMIT 1');
		$stmt->execute(array($id));
		return $stmt->fetch();
	}
	
	/**
	 * Usuwa newsa
	 *
	 * @param integer
	 * 
	 * @access static
	 * @return bool
	 */
	static public function delete($nid)
	{
		$delete = Core::db()->prepare('DELETE FROM `news_` where `id` = ?');
		$delete->execute(array($nid));
		$count  = $delete->rowCount();
		
		$delete = Core::db()->prepare('DELETE FROM `reputation_cache_` where `section` = \'news\' and `section_id` = ?');
		$delete->execute(array($nid));
				  
		return $count;
	}
	
	/**
	 * Dodaje newsa...
	 *
	 * @access public
	 * @return array
	 */
	static public function create($data)
	{
		
		if(empty($data['title']) or empty($data['content']) or strlen($data['content']) < 5)
		{
			return array(0, 'Wypełnij wszystkie pola');
		}
	
	
		$data['author_id'] = Member::logged();
		$data['time']	   = time();
		
		$last_news = Core::db()->query('SELECT `news_time` FROM `news_` order by `id` DESC LIMIT 1')->fetch();
		
		if(( $time = $data['time'] - $last_news['news_time']) < (60*Core::setting()->get('news_anty_spam_time', 20)))
		{
			$allowed_at = 60 * Core::setting()->get('news_anty_spam_time', 20)  - $time;
			$allowed_at = round($allowed_at/60, 0);
			return array(0, 'Newsy można dodawać co '. Core::setting()->get('news_anty_spam_time', 20).' minut, musisz poczekać ' . $allowed_at . ' minut');
		}
		
		$stmt = Core::db()->prepare('INSERT INTO `news_` VALUES(NULL, ?, ?, ?, ?)');
		$stmt->execute(array($data['author_id'], $data['title'], $data['content'], $data['time']));
		
		
		return array(1, 'News został dodany...');
	}
	
	/**
	 * Edytuje newsa...
	 *
	 * @param integer
	 * @param array
	 *
	 * @access public
	 * @return array
	 */
	static public function edit($id, $data)
	{
		if(!$data['title'] or strlen($data['content']) < 4)
		{
			return array(0, 'Nie wypełniłeś wszystkich pól');
		}
		
		
		$stmt = Core::db()->prepare('UPDATE `news_` set `news_title` = ?, `news_content`= ? where `id` = ?');
		$stmt->execute(array($data['title'], $data['content'], $id));
		
		if($stmt->rowCount())
		{
			return array(1, 'Zmodyfikowano newsa');
		}
		
		return array(0, 'Żadne dane newsa nie zostały zmienione...');
	}
	
	/**
	 * Reputation news +/-
	 *
	 * @param integer
	 * @param integer
	 *
	 * @access static
	 * @return array
	 */
	static public function like($news_id, $vote)
	{
		// Sprawdźmy czy użytkownik jest zalogowany
		if(!$member_id = Member::logged())
		{
			return array(0, 'Aby oddać głos musisz się zalogować');
		}
		
		$data = News::getOne($news_id);
		// Jeśli autor == użytkownik który chce polub zwracamy false
		if($data['author_id'] == $member_id && !Core::setting()->get('news_my_like', 0))
		{
			return array(0, 'Nie możesz oddać głosu na swojego newsa');
		}
		
		// Pobierzmy reputacje newsa
		$query = Core::db()->prepare('SELECT * FROM `reputation_cache_` where `section` = \'news\' and `section_id` = ? LIMIT 1');
		$query->execute(array($news_id));
		
		// Jeśli brak wpisu z reputacją dla tego newsa to go utwórzmy
		if(!$query->rowCount())
		{
			$query = Core::db()->prepare('INSERT INTO `reputation_cache_` VALUES(NULL, \'news\', ?, 0, ?)');
			$query->execute(array($news_id, 'a:0:{}'));
			
			// I powtórzmy działanie...
			return News::like($news_id, $vote);
		}
		
		// pobierzmy dane
		$data = $query->fetch();
		
		// rozpakujmy string do tablicy
		$data['rep_cache'] = unserialize($data['rep_cache']);
	
	
		// Jeśli użytkownik oddał już głos
		if(isset($data['rep_cache'][$member_id]))
		{
			return array(0, 'Już polubiłeś tego newsa');
		}
		
		// Zaznaczmy że ten użytkownik oddał głos
		$data['rep_cache'][$member_id] = array(
			'time' => time(),       // Czas kiedy użytkownik zagłosował
			'vote' => $vote ? 1 : 0 // typ głosu +/-
		);
		
		// i zapakujmy tablice do stringu
		$data['rep_cache']  = serialize($data['rep_cache']);
		
		// Zaktualizujmy reputacje newsa
		$data['rep_points'] = $vote ? $data['rep_points'] + 1 : $data['rep_points'] - 1;
		
		// Wyślimy zapytanie
		$query = Core::db()->prepare('UPDATE `reputation_cache_` SET `rep_points` = ?, `rep_cache` = ? where `id` = ?');
		$query->execute(array($data['rep_points'], $data['rep_cache'], $data['id']));
		
		// Jeśli zapytanie się powiodło i zmodyfikowano jakiś wpis
		if($query->rowCount())
		{
			return array(1, $data['rep_points']);
		}
		// Jeśli coś poszło nie tak...
		return array(0, 'Błędna ocena');
	}
	
	/**
	 * Get reputation
	 *
	 * @param integer
	 *
	 * @access static
	 * @return array
	 */
	static public function getReputation($id)
	{
		$stmt = Core::db()->prepare('SELECT * FROM `reputation_cache_` where `section` = \'news\' AND `section_id` = ?');
		$stmt->execute(array($id));
		$data = $stmt->fetch();
		
		return is_array($data) ? unserialize($data['rep_cache']) : array();
	}
}