<?php defined('SYSPATH') or die('No direct script access.');

class Payments_Pukawka
{
	/**
	 * Sprawdza poprawność kodu sms...
	 *
	 * @param  $sms    string kod sms
	 * @param  $numer  string opcjonalnie, numer na który został wysłany sms
	 * @param  $code   string opcjonalnie, treść smsa jaką wysłał użytkownik
	 *
	 * @access public
	 * @return array wynik sprawdzania SMS array(KWOTA, TRESC)
	 */
	public function check($sms, $numer = NULL, $code = NULL)
	{
		$key_api = Core::setting()->get('keyapi_pukawka');
		
		$https_query = @file_get_contents('https://admin.pukawka.pl/api/?keyapi='.$key_api.'&type=sms&code=' . $sms);

		if($https_query)
		{
			$https_query = json_decode($https_query);
			
			if(!is_object($https_query))
			{
				return array(0.0, 'Błąd API');
			}
			
			if(isset($https_query->error))
			{
				switch($https_query->error)
				{
					case 'wrong_api_type':
						$http_error = 'Błędny typ api';
					break;
					case 'wrong_api_key':
						$http_error = 'Błędny klucz API';
					break;
					case 'empty_sms_code':
						$http_error = 'Brak kodu sms';
					break;
					case 'wrong_sms_code':
						$http_error = 'Kod sms zawiera nieprawidłowe znaki.';
					break;
					case 'wrong_desc_len':
						$http_error = 'Zbyt długi opis płatności.';
					break;
					case 'internal_api_error':
						$http_error = 'Wewnętny błąd API Pukawki';
					break;
				}
				return array(0.0, $http_error);
			}
			
			if($https_query->status == 'ok')
			{
				return array($https_query->kwota, 'Portfel został doładowany');
			}
			else
			{
				return array(0.0, 'Kod sms jest nieprawidłowy');
			}
		}
		
		return array(0.0, 'Nie można nawizać połączenia z API');
	}
	
	
	/**
	 * Stawki sms
	 *
	 *
	 * @access public
	 * @return array zwraca tablice ze stawkami sms(numer, koszt, tresc)
	 */
	public function rates()
	{
		$key_api = Core::setting()->get('keyapi_pukawka');
		$rates   = file_get_contents("https://admin.pukawka.pl/api/?keyapi=$key_api&type=sms_table");
		$rates   = json_decode($rates);
		
		$_return = array();
		foreach($rates as $rate)
		{
			$_return[] = array(
				'numer'    => $rate->numer,	 // numer na jaki ma zostać wysłany sms
				'text_sms' => $rate->tresc,  // treść smsa
				'cost'	   => $rate->koszt,  // Koszt smsa
				'amount'   => $rate->wartosc // wartość jaką zostanie doładowany portfel
			);
		}
		
		return $_return;
	}
}
?>